package depGui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;

import javax.swing.JFrame;


public class DepsShow extends JFrame {
	private static final long serialVersionUID = 1L;
	private DibuixGraf<String> grafo;
	private boolean externs;
	private String incs;
	private String projectDir;
	
	public enum OutputFormat { png, jpg, dot, html, graphic }
	public enum CPPCode { startNewFile, returningFile, SystemHeaderFile, externC }
	
	public DepsShow() {
		externs=true;
		
		grafo = new DibuixGraf<String>();
		grafo.setTipus(DibuixGraf.Tipus.circular);
		getContentPane().add(grafo, BorderLayout.CENTER);
		pack();
	}
	
	public void setGraphType(DibuixGraf.Tipus t)
	{
		grafo.setTipus(t);
	}
	
	public void setProjectDir(String s) {
		if(!s.endsWith("/"))
			s+= "/";
		projectDir=s;
	}
	
	public void setExterns(boolean e) { externs=e; }
	
	class CPPLine {
		public CPPLine() { key=""; line=-1; path=""; codes=new TreeSet<CPPCode>(); }
		public CPPLine(String _key, int _line, String _path, Set<CPPCode> _code) { key=_key; line=_line; path=_path; codes=_code; }
		public String key;
		public int line;
		public String path;
		public Set<CPPCode> codes;
		public String toString() {
			String code="", flags="";
			for(CPPCode c : codes) {
				switch(c) {
					case startNewFile:
						code+="start";
						break;
					case returningFile:
						code+="end";
						break;
					case SystemHeaderFile:
						code+="sys header; ";
						break;
					case externC:
						code+="externC; ";
						break;
					default:
						code+="nuze; ";
						break;
				}
			}
			return code+ " -> " +path+ " "+flags;
		}
	}
	
	private CPPLine parseGnuCppLine(String s) {
		CPPLine ret = null;
		if(s.charAt(0)== '#') {
			ret = new CPPLine();
			int i=1, step=0, bracket;
			boolean finished=false;
			String param;
			for(step=0; !finished; step++) {
				param="";
				bracket=0;
				for(; i<s.length() && s.charAt(i)==' '; i++) {}
				for(; i<s.length() && (bracket>0 || s.charAt(i)!=' '); i++) {
					char act = s.charAt(i); 
					if(act=='<')
						bracket++;
					else if(act=='>')
						bracket--;
					
					param += act;
				}
				
				param = param.trim();
				if(param.isEmpty())
					finished=true;
				else switch(step) {
					case 0:
						if(param.charAt(0)>'a')
							ret.key = param;
						else
							ret.line = Integer.parseInt(param);
						break;
					case 1:
						ret.path = param;
						break;
					default:
					case 2:
						if(ret.line>0) {
							switch(Integer.parseInt(param)) {
								case 1:
									ret.codes.add(CPPCode.startNewFile);
									break;
								case 2:
									ret.codes.add(CPPCode.returningFile);
									break;
								case 3:
									ret.codes.add(CPPCode.SystemHeaderFile);
									break;
								case 4:
									ret.codes.add(CPPCode.externC);
									break;
								default:
									System.err.println("codi "+param);
									break;
							}
						}
						break;
				}
			}
		}
		
		ret.path= unquote(ret.path);
		if(!ret.path.isEmpty() && ret.path.startsWith(projectDir)) {
			ret.path = ret.path.substring(projectDir.length(), ret.path.length());
		}
		return ret;
	}
	
	public void setPath(String path, String cppParams) {
		if(projectDir==null)
			setProjectDir(path);
		
		grafo.clear();
		grafo.addVertex("Project", 1, DibuixGraf.TipusNode.arrel);
		File dir = new File(path);
	    String[] children = dir.list();
	    
	    FileInputStream idf;
	    if(cppParams!=null) {
	        try {
	                idf = new FileInputStream (cppParams);       //obrim canal per llegir un fitxer
	        } catch (FileNotFoundException e) {
	                System.out.println("error en llegir la pagina: " + e.toString());
	                return;
	        }
	        
	        InputStreamReader isr = new InputStreamReader(idf);
	        BufferedReader contingut= new BufferedReader(isr);

	        try {
				incs = contingut.readLine();
			} catch (IOException e) {
				System.out.println("Error en llegir includes.");
				e.printStackTrace();
			}
	    }
        if(incs==null)
        	incs = "-I. -I"+path;
	    
	    for(String f : children) {
	    	if(f.endsWith(".cpp") || f.endsWith(".c")) {
	    		if(f.startsWith(projectDir))
					f = f.substring(projectDir.length(), f.length()-projectDir.length());
	    		
	    		grafo.addVertex(f, 1);
	    		grafo.addAresta("Project", f);
		    	System.out.println("reading: "+f);
		    	readIncludeList(path+"/"+f);
	    	}
	    }
	    grafo.camiMinim();
	}
	
	private String unquote(String s) {
		int ini=0, fi=s.length();
		if(s.charAt(ini)=='"')
			ini++;
		if(s.charAt(fi-1)=='"')
			fi--;
		return s.substring(ini, fi);
	}
	
	private void readIncludeList(String path) {
		String encoding="UTF8";
		Process p=null;
		try {
			p = Runtime.getRuntime().exec("cpp "+path+" "+incs);
			//System.out.println("cpp "+path+" "+incs);
		} catch (IOException e) {
			System.out.println("error en executar proces:" + e.toString());
			e.printStackTrace();
		}
		
		InputStreamReader isr=null;
		try {
			isr = new InputStreamReader(p.getInputStream(), encoding);
		} catch (UnsupportedEncodingException e1) {
			e1.printStackTrace();
		}
		BufferedReader contingut= new BufferedReader(isr);
		String aux=null;
		try {
			aux = contingut.readLine();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		Stack<String> currentFile = new Stack<String>(); //FIXME: Can turn it to String
		int depth=0, waitdepth=-1;
		while(aux!=null){
			if(aux.isEmpty() || aux.charAt(0)!='#') { //We don't need the code
				try {
					aux = contingut.readLine();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				continue;
			}
			
			CPPLine out = parseGnuCppLine(aux);
			if(currentFile.isEmpty() && out!=null)
				currentFile.push(out.path);
			
			if(out!=null && !out.codes.isEmpty()) {
				boolean add=false;
				if(waitdepth>depth) //Waiting for the next appropiated level
					waitdepth=depth;
				else {
					if(!grafo.hasVertex(out.path) ) {
						waitdepth=-1;
						DibuixGraf.TipusNode t = DibuixGraf.TipusNode.normal;
						if(out.codes.contains(CPPCode.SystemHeaderFile) || out.path.startsWith("/")) {
							t = DibuixGraf.TipusNode.especial;
							if(externs)
								add=true;
						} else
							add=true;
						
						if(add)
							grafo.addVertex(out.path, 1, t);
					}
					
					if(out.codes.contains(CPPCode.startNewFile))
						grafo.addAresta(currentFile.peek(), out.path);
				}
				
				if(out.codes.contains(CPPCode.startNewFile)) {
					depth++;
					currentFile.push(out.path);
				} else if(out.codes.contains(CPPCode.returningFile)) {
					depth--;
					currentFile.pop();
				}
			}
			try {
				aux = contingut.readLine();
			} catch (IOException e) {
				System.out.println("error");
				e.printStackTrace();
			}
		}
	}
	
	public void export(String path, OutputFormat oform) {
		switch(oform) {
		case png:
			grafo.export(path, "png", Color.red);
			break;
		case jpg:
			grafo.export(path, "jpg", Color.blue);
			break;
		case dot://TODO: dot export for graphviz
			break;
		case html:
			grafo.exportHtml(path, "white", true);
			break;
		default:
			break;
		}
	}

	public void setRootShow(boolean b) {
		grafo.setRootShow(b);
	}
}
